#!/bin/bash
# IPC - Custom Scripts - fail2ban

f2bSAP="/usr/bin/fail2ban-server"
f2bJC="/etc/fail2ban/jail.conf"
f2bFcmd="/etc/fail2ban/filter.d/cccam-command.conf"
f2bFsign="/etc/fail2ban/filter.d/cccam-signature.conf"
f2bFlogin="/etc/fail2ban/filter.d/cccam-login.conf"
f2bFillegal="/etc/fail2ban/filter.d/cccam-illegal.conf"
FILTER_cmd=""
FILTER_sign=""
FILTER_2login=""
FILTER_illegal=""
FILTER_name_cmd="Bad_command"
FILTER_name_sign="Signature_failed"
FILTER_name_2login="Double_login"
FILTER_name_illegal="illegal_user"
OptimalFILTER="2login sign"
OptimalLIST=""
ScriptVer="1.00"

. /var/emu/script/functions.sh
readipccfg
readcccamcfg

if [ -z "$CCSPORT" ]; then
	CCSPORT=$(grep -i "SERVER LISTEN PORT" $CCCFG | cut -d ":" -f2 | sed -e 's/ //g' | sed -e 's/\r//g')
fi


if [ -z "$OptimalLIST" ]; then
	for OF in $OptimalFILTER; do
		if [ "$OF" = "cmd" ]; then
			OptimalLIST="$OptimalLIST $FILTER_name_cmd"
		elif [ "$OF" = "sign" ]; then
			OptimalLIST="$OptimalLIST $FILTER_name_sign"
		elif [ "$OF" = "2login" ]; then
			OptimalLIST="$OptimalLIST $FILTER_name_2login"
		elif [ "$OF" = "illegal" ]; then
			OptimalLIST="$OptimalLIST $FILTER_name_illegal"
		fi
	done
	OptimalLIST=$(echo $OptimalLIST | sort | sed 's/ /, /g' | tr "_" " ")
fi


clear
echo -e "IPC $IPCVERSION: \033[1;37mCCcam fail2ban v$ScriptVer\033[0m                          $(date +"%d.%m.%Y %H:%M")"
echo "=================================================  =============================="
echo ""
echo " fail2ban ist ein kleines Tool, welches unter Linux Logs durchsucht und bei"
echo " bestimmten Inhalten entsprechende Aktionen ausfuehren kann. Es kann z.B."
echo " Spambots und Vulnerability Scanner dynamisch erkennen und aussperren."
echo " Das bringt einige Vorteile mit sich:"
echo ""
echo " * Logs koennen daran gehindert werden, durch Attacken Speicherplatz zu fressen"
echo "   bis keiner mehr da ist (denn dann wird es erst richtig lustig)"
echo " * ScriptKidis koennen sich schoen die Milchzaehne an dem Server ausbeissen."
echo " * Bots, die nach bekannten Sicherheitsluecken suchen koennen leicht gesperrt"
echo "   werden."
echo " * Brutefoce-Attacken koennen leicht und automatisch geblockt werden."
echo ""
echo " fail2ban fuer CCcam - schuetzt vor:"
echo "   Double login, Bad command, Signature failed, Illegal user"
echo " und sperrt die Verursacher-IP ueber einen definierbaren Zeitraum (30min)"
echo ""
echo -e " \033[1;37mOptimale CCcam fail2ban Filter Einstellung\033[0m:"
echo "    $OptimalLIST"
echo ""
echo "-------------------------------------------------------------------------------"
echo ""
echo -en "\033[1;33m[I]nstallieren  |  [O]ptimale Installation  |  []Beenden \033[0m"
read key
case $key in
	[Ii])
		UseOptimal=0
		echo ""
	;;
	[oO0])
		UseOptimal=1
		echo ""
	;;
	*)
		echo ""
		echo "-------------------------------------------------------------------------------"
		exit
	;;
esac

### ---------------------------------------------------------

proc_CurrentCCcamVersion() {
	cd /tmp ; rm -f index.html
	weblogon
	wget -q http://$WUP$CCURL:$CCPORT/$1
	CCV=$(cat index.html | grep "Welcome" | sed 's/^.*\( [0-9.]* \).*$/\1/')
	rm -f index.html
}
proc_CurrentCCcamVersion
if [ -z "$CCV" ]; then
	echo -e "\033[1;31mFuer fail2ban wird ein laufendes CCcam benoetigt! Abbruch!\033[0m"
	echo ""
	echo "-------------------------------------------------------------------------------"
	exit 1
fi
CCV2=$(echo $CCV | tr -d ".")

if [ ! -f "$f2bSAP" ]; then
	echo -n "Installiere fail2ban.."
	apt-get update >/dev/null
	echo ".."
	apt-get install -fy fail2ban
	echo ""
fi

### ---------------------------------------------------------

proc_jail() {
	BACKUPjail=${f2bJC}.bak
	if [ ! -f "$BACKUPjail" ]; then
		cp -f $f2bJC $BACKUPjail
	fi
	#echo '' >> $f2bJC
	echo '[cccam_sign]' >> $f2bJC
	if [ ! -z "$FILTER_sign" ]; then
		echo 'enabled = true' >> $f2bJC
	else
		echo 'enabled = false' >> $f2bJC
	fi
	echo "port = ${CCSPORT}" >> $f2bJC
	echo 'filter = cccam-signature' >> $f2bJC
	echo 'logpath = /var/log/syslog' >> $f2bJC
	echo 'bantime = 1800' >> $f2bJC
	echo 'maxretry = 10' >> $f2bJC
	#echo '' >> $f2bJC
	echo '[cccam_badcmd]' >> $f2bJC
	if [ ! -z "$FILTER_cmd" ]; then
		echo 'enabled = true' >> $f2bJC
	else
		echo 'enabled = false' >> $f2bJC
	fi
	echo "port = ${CCSPORT}" >> $f2bJC
	echo 'filter = cccam-command' >> $f2bJC
	echo 'logpath = /var/log/syslog' >> $f2bJC
	echo 'bantime = 1800' >> $f2bJC
	echo 'maxretry = 10' >> $f2bJC
	#echo '' >> $f2bJC
	echo '[cccam_2login]' >> $f2bJC
	if [ ! -z "$FILTER_2login" ]; then
		echo 'enabled = true' >> $f2bJC
	else
		echo 'enabled = false' >> $f2bJC
	fi
	echo "port = ${CCSPORT}" >> $f2bJC
	echo 'filter = cccam-login' >> $f2bJC
	echo 'logpath = /var/log/syslog' >> $f2bJC
	echo 'bantime = 1800' >> $f2bJC
	echo 'maxretry = 10' >> $f2bJC
	#echo '' >> $f2bJC
	echo '[cccam_illegal]' >> $f2bJC
	if [ ! -z "$FILTER_illegal" ]; then
		echo 'enabled = true' >> $f2bJC
	else
		echo 'enabled = false' >> $f2bJC
	fi
	echo "port = ${CCSPORT}" >> $f2bJC
	echo 'filter = cccam-illegal' >> $f2bJC
	echo 'logpath = /var/log/syslog' >> $f2bJC
	echo 'bantime = 1800' >> $f2bJC
	echo 'maxretry = 10' >> $f2bJC
}

## Filter: cccam-command.conf
proc_filter_cmd() {
	if [ -f "$f2bFcmd" ]; then
		rm -f $f2bFcmd
	fi
	echo '[Definition]' >> $f2bFcmd
	if [ "$CCV2" = "2011" ]; then
		echo 'failregex = CCcam: deleting client <HOST>.*, read result -1' >> $f2bFcmd
	elif [ "$CCV2" = "211" ]; then
		echo 'failregex = CCcam: deleting client <HOST>.*, bad command' >> $f2bFcmd
	else
		echo 'failregex = CCcam: kick <HOST>.*, bad command' >> $f2bFcmd
	fi
	echo 'ignoreregex =' >> $f2bFcmd
}

## Filter: cccam-signature.conf
proc_filter_sign() {
	if [ -f "$f2bFsign" ]; then
		rm -f $f2bFsign
	fi
	echo '[Definition]' >> $f2bFsign
	if [ "$CCV2" = "2011" ] || [ "$CCV2" = "211" ]; then
		echo 'failregex = CCcam: deleting client <HOST>, signature failed' >> $f2bFsign
	else
		echo 'failregex = CCcam: kick <HOST>, signature failed' >> $f2bFsign
	fi
	echo 'ignoreregex =' >> $f2bFsign
}

## Filter: cccam-login.conf
proc_filter_2login() {
	if [ -f "$f2bFlogin" ]; then
		rm -f $f2bFlogin
	fi
	echo '[Definition]' >> $f2bFlogin
	if [ "$CCV2" = "2011" ] || [ "$CCV2" = "211" ]; then
		echo 'failregex = CCcam: client .* already connected, remove stale connection \(<HOST>\)' >> $f2bFlogin
	else
		echo 'failregex = CCcam: double login .*, .* \(<HOST>\)' >> $f2bFlogin
	fi
	echo 'ignoreregex =' >> $f2bFlogin
}

## Filter: cccam-illegal.conf
proc_filter_illegal() {
	if [ -f "$f2bFillegal" ]; then
		rm -f $f2bFillegal
	fi
	echo '[Definition]' >> $f2bFillegal
	if [ "$CCV2" = "2011" ] || [ "$CCV2" = "211" ]; then
		echo 'failregex = CCcam: illegal user .* from <HOST>' >> $f2bFillegal
	else
		echo 'failregex = CCcam: illegal user .* from <HOST>' >> $f2bFillegal
	fi
	echo 'ignoreregex =' >> $f2bFillegal
}

function renew_jailconf() {
	FILE=$f2bJC
	BACKUPf=${FILE}.bak
	# backup der alten jail.conf erstellen
	if [ ! -f "$BACKUPf" ]; then
		cp -f $FILE $BACKUPf
	fi
	rm -f $FILE >/dev/null
	# was wird geskipped
	SkipLines=$(sed -ne '/\[cccam_.*\]/,+6=' $BACKUPf | sort)
	x=0
	while read line; do
		x=$(expr $x + 1)
		DoIt=1
		for linenum in $SkipLines; do
			if [ "$linenum" = "$x" ]; then
				DoIt=0
			fi
		done
		if [ "$DoIt" = "1" ]; then
			echo "$line" >> $FILE
		fi
	done < $BACKUPf
}

### ---------------------------------------------------------

echo "Einrichten der Filter.."

## OptimalFILTER installieren?
if [ "$UseOptimal" = 1 ]; then
	for OF in $OptimalFILTER; do
		if [ "$OF" = "cmd" ]; then
			FILTER_cmd=1
			proc_filter_cmd
		elif [ "$OF" = "sign" ]; then
			FILTER_sign=1
			proc_filter_sign
		elif [ "$OF" = "2login" ]; then
			FILTER_2login=1
			proc_filter_2login
		elif [ "$OF" = "illegal" ]; then
			FILTER_illegal=1
			proc_filter_illegal
		fi
	done
	echo ""
else
	echo ""
	echo -en "\033[1;33m'Double login' Filter aktivieren? [J]a | []Nein \033[0m"
	read key
	case $key in
		[JjYy]|ja|Ja|yes)
			echo ""
			proc_filter_2login
			FILTER_2login=1
		;;
		*)
			echo ""
		;;
	esac
	echo -en "\033[1;33m'Bad Command' Filter aktivieren? [J]a | []Nein \033[0m"
	read key
	case $key in
		[JjYy]|ja|Ja|yes)
			echo ""
			proc_filter_cmd
			FILTER_cmd=1
		;;
		*)
			echo ""
		;;
	esac
	echo -en "\033[1;33m'Signature failed' Filter aktivieren? [J]a | []Nein \033[0m"
	read key
	case $key in
		[JjYy]|ja|Ja|yes)
			echo ""
			proc_filter_sign
			FILTER_sign=1
		;;
		*)
			echo ""
		;;
	esac
	echo -en "\033[1;33m'illegal user' Filter aktivieren? [J]a | []Nein \033[0m"
	read key
	case $key in
		[JjYy]|ja|Ja|yes)
			echo ""
			proc_filter_illegal
			FILTER_illegal=1
		;;
		*)
			echo ""
		;;
	esac
	echo "-------------------------------------------------------------------------------"
	echo ""
fi

### ---------------------------------------------------------

## uebersicht, welche FILTER sind aktiviert/deaktiviert...
_AKT="\033[0;32maktiviert\033[0m"
_DKT="\033[1;31mdeaktiviert\033[0m"
FTRDIS_cmd=$_DKT
FTRDIS_sign=$_DKT
FTRDIS_2login=$_DKT
FTRDIS_illegal=$_DKT
if [ "$FILTER_cmd" = 1 ]; then
	FTRDIS_cmd=$_AKT
fi
if [ "$FILTER_sign" = 1 ]; then
	FTRDIS_sign=$_AKT
fi
if [ "$FILTER_2login" = 1 ]; then
	FTRDIS_2login=$_AKT
fi
if [ "$FILTER_illegal" = 1 ]; then
	FTRDIS_illegal=$_AKT
fi
echo -e "Double login:     $FTRDIS_2login"
echo -e "Bad command:      $FTRDIS_cmd"
echo -e "Signature failed: $FTRDIS_sign"
echo -e "illegal user:     $FTRDIS_illegal"

### ---------------------------------------------------------

echo ""
echo -n "$f2bJC wird fuer CCcam ${CCV} aktualisiert.."
renew_jailconf
echo -n ".."
sleep 1
proc_jail
echo " ok"

### ---------------------------------------------------------

echo ""
/etc/init.d/fail2ban restart
echo ""
echo "fail2ban Installation fuer CCcam (port ${CCSPORT}) abgeschlossen!"
echo "-------------------------------------------------------------------------------"

exit 0
