#!/bin/bash
# IPC - Functions - 32bit
#
#
# set -x #debug

#++++++++++++++++++++++++++++++ VARIABLES ++++++++++++++++++++++++++++++

IPCVERSION="11.3"

AUTORUN=/etc/rc.local
IPCCONFIG="/var/emu/conf/ipc.cfg"
CAMCONFIGS=/var/etc
CAMBINS=/var/emu
CAMLOGS=/var/log/ipc
CAMJOBS=/etc/crontab

CCNAME="CCcam"
CCVERSION="CCcam.x86"
CCURL="localhost"
CCWWW="/var/www/ipc"
CCCFG="$CAMCONFIGS/CCcam.cfg"
CCCFGNEW="$CAMCONFIGS/CCcam.cfg.new"
CCCFGBAK="$CAMCONFIGS/CCcam.cfg.old"
CCCHANGELOG="$CAMLOGS/CCcam-changes.log"

OSNAME="OScam"
OSVERSION="oscam.x86"
OSBIN=$CAMBINS/oscam/$OSVERSION
OSCFG="$CAMCONFIGS/oscam.conf"
OSSERVER="$CAMCONFIGS/oscam.server"
OSUSER="$CAMCONFIGS/oscam.user"
OSERVICES="$CAMCONFIGS/oscam.services"
OSRVID="$CAMCONFIGS/oscam.srvid"

NCSNAME="NewCS"
NCSVERSION="newcs.i686"
NCSBIN=$CAMBINS/newcs/$NCSVERSION
NCSXML="$CAMCONFIGS/newcs.xml"
NCSLOG="$CAMLOGS/newcs.log"

BACKUPDIR="/var/backups/ipc"
TMPDIR="/var/emu/tmp/script"
TMPFILE1="$TMPDIR/CCcam1.tmp"
TMPFILE2="$TMPDIR/CCcam2.tmp"
TIMESTAMP=`date +"%Y%m%d_%H%M%S"`
TIMESTAMP2=`date +"%Y-%m-%d %H:%M"`

INFOPHP_CONFIG=/var/www/ipc/infophp/config.php
INFOPHP_WORKPATH='/var/emu/tmp/infophp/'
INFOPHP_UPDATEFROMBUTTON='true'
INFOPHP_FULLRESHARE='true'
INFOPHP_COUNTRYWHOIS='true'

IPURL="http://ipc.pebkac.at/ipc"
FMURL="http://ipc.pebkac.at/forum"

SHSETUP=ipcsetup.sh

#++++++++++++++++++++++++++++++ VARIABLES ++++++++++++++++++++++++++++++

#++++++++++++++++++++++++++++++ FUNCTIONS ++++++++++++++++++++++++++++++

# .webaccess neu schreiben
NewWebAccess() {
  htpasswd -bc /var/emu/conf/.webaccess $1 $2
}

# Tempdir erstellen, Rechte setzen, leeren
tempdir() {
  #echo "Function: tempdir"
  [ ! -d $TMPDIR ] && mkdir -p $TMPDIR && chmod 777 -R $TMPDIR  > /dev/null 2>&1
  cd $TMPDIR
  rm -f $TMPDIR/*  > /dev/null 2>&1
  cd $TMPDIR
}

# Rechte setzen
setrights() {
  OWNER="www-data:www-data"
  chown -R $OWNER /var/etc /var/keys > /dev/null 2>&1
  #CCcam.cfg - Rechte und Besitzer
  chmod 755 /var/etc > /dev/null 2>&1
  chmod 755 $CCCFG > /dev/null 2>&1
  chown $OWNER $CCCFG > /dev/null 2>&1
  #CCcam key files
  chmod 755 /var/keys/CCcam.* >/dev/null 2>&1
  chown $OWNER /var/keys/CCcam.* >/dev/null 2>&1
  #OScam config files
  chmod 755 $OSCFG >/dev/null 2>&1
  chmod 755 $OSSERVER >/dev/null 2>&1
  chmod 755 $OSUSER >/dev/null 2>&1
  chmod 755 $OSERVICES >/dev/null 2>&1
  chmod 755 $OSRVID >/dev/null 2>&1
  chown $OWNER $OSCFG >/dev/null 2>&1
  chown $OWNER $OSSERVER >/dev/null 2>&1
  chown $OWNER $OSUSER >/dev/null 2>&1
  chown $OWNER $OSERVICES >/dev/null 2>&1 
  chown $OWNER $OSRVID >/dev/null 2>&1
  #NewCS config file
  chmod 755 $NCSXML >/dev/null 2>&1
  chown $OWNER $NCSXML >/dev/null 2>&1
  #EMU
  chmod 755 -R /var/emu > /dev/null 2>&1
  chmod 777 -R /var/emu/tmp > /dev/null 2>&1
  chown $OWNER -R /var/emu/tmp > /dev/null 2>&1
  #Changelog - Erstellen, Rechte, Besitzer
  [ ! -f $CCCHANGELOG ] && echo "" > $CCCHANGELOG > /dev/null 2>&1
  chmod 755 $CCCHANGELOG > /dev/null 2>&1
  chown $OWNER $CCCHANGELOG > /dev/null 2>&1
  #InfoPHP Tempdir und Rechte
  [ ! -d $INFOPHP_WORKPATH ] && mkdir -p $INFOPHP_WORKPATH > /dev/null 2>&1
  chmod 777 -R $INFOPHP_WORKPATH > /dev/null 2>&1
  chown $OWNER -R $INFOPHP_WORKPATH > /dev/null 2>&1
  chown $OWNER $INFOPHP_CONFIG > /dev/null 2>&1
}

# Paket Konfiguration auslesen
readipccfg() {
  #echo "Function: readipccfg"
  source $IPCCONFIG
}

# CCcam.cfg Konfiguration auslesen
readcccamcfg() {
  #echo "Function: readcccamcfg"
  CCUSER=$(grep -i "WEBINFO USERNAME" $CCCFG | cut -d ":" -f2 | sed -e 's/ //g' | sed -e 's/\r//g')
  CCPASS=$(grep -i "WEBINFO PASSWORD" $CCCFG | cut -d ":" -f2 | sed -e 's/ //g' | sed -e 's/\r//g')
  CCPORT=$(grep -i "WEBINFO LISTEN PORT" $CCCFG | cut -d ":" -f2 | sed -e 's/ //g' | sed -e 's/\r//g')
  CCCHAN=$(grep -i "CHANNELINFO FILE" $CCCFG | cut -d ":" -f2 | sed -e 's/ //g' | sed -e 's/\r//g')
  CCPROV=$(grep -i "PROVIDERINFO FILE" $CCCFG | cut -d ":" -f2 | sed -e 's/ //g' | sed -e 's/\r//g')
  CCSPORT=$(grep -i "SERVER LISTEN PORT" $CCCFG | cut -d ":" -f2 | sed -e 's/ //g' | sed -e 's/\r//g')
  [ "$CCSPORT" = "" ] && CCSPORT="12000"
  [ "$CCPORT" = "" ] && CCPORT="16001"
  #echo "CCUSER: $CCUSER"
  #echo "CCPASS: $CCPASS"
  #echo "CCPORT: $CCPORT"
}

# Teste ob CCcam laeuft
cccamrun() {
  #echo "Function: cccamrun"
  PID="$(pgrep -x $CCVERSION)"
  if [[ ! -z $PID ]] ; then
    dummy="1"  
    #echo "CCcam laeuft"
  else
    echo "CCcam ist nicht gestartet"
    exit 1
  fi
}

# Weblogin testen
weblogon() {
  #echo "Function: weblogon"
  WUP=""
  [ "$CCUSER" != "" ] && WUP="$CCUSER:$CCPASS@"
  CCWEBIF="http://$WUP$CCURL:$CCPORT"
  WEBLOGIN=$(wget -q -O - $CCWEBIF)
  [[ -z $WEBLOGIN ]] && clear && echo "CCcam Weblogon ERROR!" && exit 1
}

# E-Mail senden
mailsend() {
  readipccfg
  echo "Sende E-Mail..."
  sendEmail -f $SMTPFROM -t $SMTPTO -u $1 -m $2 -s $SMTPSERVER -xu $SMTPUSER -xp $SMTPPASS
}

# InfoPHP Konfiguration schreiben
infophpconfwrite() {
  echo "<?php" >> $TMPFILE1
  echo "" >> $TMPFILE1
  echo "// CONFIG CCcamInfoPhp" >> $TMPFILE1
  echo "// Diese Config wird automatisch beim schliessen von [cccam cfg] erzeugt!" >> $TMPFILE1
  echo "" >> $TMPFILE1
  echo '$CCCamWebInfo[] = array("'$CCURL'","'$CCPORT'","'$CCUSER'","'$CCPASS'"); //Hier die Infos aus der CCcam.cfg eintragen' >> $TMPFILE1
  echo "" >> $TMPFILE1
  echo '$work_path = "'$INFOPHP_WORKPATH'"; // Verzeichnis der temporaeren InfoPhp Dateien' >> $TMPFILE1
  echo '$update_from_button = '$INFOPHP_UPDATEFROMBUTTON'; // Update Button anzeigen?' >> $TMPFILE1
  echo '$fullReshare = '$INFOPHP_FULLRESHARE'; // Reshareanzahl der Pairkarten anzeigen?' >> $TMPFILE1
  echo '$country_whois = '$INFOPHP_COUNTRYWHOIS'; // Laenderkennzeichen anzeigen?' >> $TMPFILE1
  echo '$password = "'$INFOPHP_PASS'"; // InfoPhp Loginseite Kennwort' >> $TMPFILE1
  echo "" >> $TMPFILE1
  echo "?>" >> $TMPFILE1
  if [ -f $INFOPHP_CONFIG ] ; then
    cp -f $TMPFILE1 $INFOPHP_CONFIG
    echo ""
    echo -e "\033[1;37mInfoPHP Konfiguration wurde uebertragen\033[0m"
  else
    echo -e "\033[1;31m$INFOPHP_CONFIG existiert nicht!\033[0m"
  fi
  rm -f $TMPDIR/*
}


