#!/bin/bash
# IPC - Backup
# (c)SEG74
# fix: CreatePackList

. /var/emu/script/functions.sh
readipccfg
[ -z $BACKUPDIR ] && clear && echo -e "\033[1;31mVariablen Konfigurationsfehler!\033[0m" && exit 1
[ -z $CAMLOGS ] && clear && echo -e "\033[1;31mVariablen Konfigurationsfehler!\033[0m" && exit 1
[ -z $TIMESTAMP ] && clear && echo -e "\033[1;31mVariablen Konfigurationsfehler!\033[0m" && exit 1
[ -z $IPCCONFIG ] && clear && echo -e "\033[1;31mIPC - Paket Konfigurationsfehler!\033[0m" && exit 1
[ -z $CAMCONFIGS ] && clear && echo -e "\033[1;31mIPC - Paket Konfigurationsfehler!\033[0m" && exit 1
[ -z $BACKUPMAIL ] && clear && echo -e "\033[1;31mIPC - Paket Konfigurationsfehler!\033[0m" && exit 1

# Variablen

LOGFILE=IPC-Backup.log
MOUNTPATH=/mnt/backup
EXCLUDES="*.tar"

clear
echo -e "IPC $IPCVERSION: \033[1;37mBackup\033[0m                                              $(date +"%d.%m.%Y %H:%M")"
echo "==============================================================================="
echo ""

# -------------------------------------------------------------------------------------------------------------------------
# Logfile erstellen
# -------------------------------------------------------------------------------------------------------------------------
echo "Backupscript : $0" > $CAMLOGS/$LOGFILE
echo "Startzeit    : $TIMESTAMP2" >> $CAMLOGS/$LOGFILE
echo "Parameter    : 1=$1, 2=$2, 3=$3, 4=$4" >> $CAMLOGS/$LOGFILE

# -------------------------------------------------------------------------------------------------------------------------
# Ueberpruefe sendEmail
# -------------------------------------------------------------------------------------------------------------------------
if [ $BACKUPMAIL == "1" ]; then
	if [ ! -f /usr/bin/sendEmail ] ; then
		echo -e "\033[1;31mMailbenachrichtigung wurde aktiviert, jedoch ist sendEmail nicht installiert!\033[0m"
		echo ""
		echo "-------------------------------------------------------------------------------"
	fi
fi

# -------------------------------------------------------------------------------------------------------------------------
# Ueberpruefe Parameterangaben, setze Variablen
# -------------------------------------------------------------------------------------------------------------------------
case "$1" in
	full)
		BACKUPART=Fullbackup
		BACKUPDIR=$BACKUPDIR/full
		BKPFILE=IPC-Fullbackup-$TIMESTAMP.tar.gz
		BKPDELO=IPC-Fullbackup-*.*
		SOURCE="${CAMCONFIGS#?} etc/crontab etc/rc.local etc/ddclient.conf root/.bashrc var/emu var/keys var/www/ipc"
		# Erstelle Backupdir
		[ ! -d $BACKUPDIR ] && mkdir -p $BACKUPDIR
	;;
	conf)
		BACKUPART=Konfigurationsbackup
		BACKUPDIR=$BACKUPDIR/conf
		BKPFILE=IPC-Confbackup-$TIMESTAMP.tar.gz
		BKPDELO=IPC-Confbackup-*.*
		SOURCE="${CAMCONFIGS#?} ${IPCCONFIG#?} etc/crontab etc/rc.local etc/ddclient.conf root/.bashrc var/keys"
		# Erstelle Backupdir
		[ ! -d $BACKUPDIR ] && mkdir -p $BACKUPDIR
	;;
	*)
		PARAMERROR=1
	;;
esac

if [ -n "$2" ]; then
	if [[ $2 =~ [lL] ]]; then
		LOKBACKUP=1
		DESTINS="$DESTINS Lokal.."
	else
		LOKBACKUP=0
	fi

	if [[ $2 =~ [nN] ]]; then
		LANBACKUP=1
		DESTINS="$DESTINS Netzwerk.."
		[ ! -d $MOUNTPATH ] && mkdir -p $MOUNTPATH
		if mount | grep -q "on $MOUNTPATH"; then
			echo "$MOUNTPATH schon verbunden, wird aufgeloest.."
			umount $MOUNTPATH
		fi

		if [ $LANTYP == "nfs" ]; then
			# Erstelle NFS Mountpath
			# Verbindungstest
			mount -t nfs $LANSHARE $MOUNTPATH >> $CAMLOGS/$LOGFILE
			if [ $? -ne 0 ]; then
				echo -e "\033[1;31mMounten ist fehlgeschlagen!\033[0m"
				echo " -> nfs-common installiert?"
				echo " -> portmap installiert?"
				echo ""
				echo -e "\033[1;37mLies in der Hilfe, Thema: Mounten\033[0m"
				echo ""
				exit 1
			fi
		else
			# Erstelle CIFS Mountpath
			# Verbindungstest
			mount -t cifs -o username=$LANUSER,password=$LANPASS, $LANSHARE $MOUNTPATH >> $CAMLOGS/$LOGFILE
			if [ $? -ne 0 ]; then
				echo -e "\033[1;31mMounten ist fehlgeschlagen!\033[0m"
				echo " -> smbfs installiert?"
				echo " -> Zugangsdaten richtig?"
				echo ""
				echo -e "\033[1;37mLies in der Hilfe, Thema: Mounten\033[0m"
				echo ""
				exit 1
			fi
		fi
	else
		LANBACKUP=0
	fi

	if [[ $2 =~ [fF] ]]; then
		FTPBACKUP=1
		DESTINS="$DESTINS FTP.."
	else
		FTPBACKUP=0
	fi
else
	PARAMERROR=1
fi

if [ -z "${3//[0-9]}" ]; then
	dummy=1
else
	PARAMERROR=1
fi

if [ -z "$4" ]; then
	GPGPASS=1
else
	GPGPASS=0
fi

#exit

if [ "$PARAMERROR" == "1" ] ; then
	echo -e "\033[1;31mDer Backupbefehl muss mit folgenden Parametern aufgerufen werden!\033[0m"
	echo ""
	echo " Parameter1 : full/conf      Full- oder Konfigurationsbackup"
	echo " Parameter2 : l/n/f          Lokal, Netzwerk, FTP"
	echo " Parameter3 : X (Tage)       Loescht Backups aelter als X Tage, 0=deaktiviert"
	echo " Parameter4 : password       GPG-Verschluesselungskennwort"
	echo ""
	echo -e "Beispiel1: Konfigurationsbackup Lokal    : \033[0;32mipc backup conf l\033[0m"
	echo -e "Beispiel2: Fullbackup verschluesselt FTP : \033[0;32mipc backup full f 0 geheim\033[0m"
	echo ""
	echo "Parameter1 und Parameter2 sind zwingend erforderlich!"
	echo -e "\033[1;37mMehr zum Thema Backup steht in der Hilfe der Administrationsseite\033[0m"
	echo ""
	echo "-------------------------------------------------------------------------------"
	echo ""
	# LOG
	echo "Parameter !  : EXIT" >> $CAMLOGS/$LOGFILE
	exit 1
else
	if [ -n "$4" ]; then
		echo "Erstelle geschuetztes $BACKUPART nach$DESTINS"
	else
		echo "Erstelle $BACKUPART nach$DESTINS"
	fi
	echo ""
fi

#
# FIX: anhand $SOURCE die zu packenden Files/Ordner verifizieren
#
function CreatePackList() {
	PACKLIST=""
	for src in $SOURCE; do
		Valid=0
		if [ -d "$src" ] || [ -f "$src" ]; then
			Valid=1
		fi
		if [ "$Valid" = 1 ]; then
			PACKLIST="$PACKLIST $src"
		fi
	done
	SOURCE=$(echo "$PACKLIST" | sort)
}

# -------------------------------------------------------------------------------------------------------------------------
# Backup Lokal (wird immer temporr erstellt, Lokal immer ohne GPG-Schutz!)
# -------------------------------------------------------------------------------------------------------------------------
cd /
# LOG
echo "" >> $CAMLOGS/$LOGFILE
CreatePackList
echo ":Backup LOKAL : Start  -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE
tar cfvz $BACKUPDIR/$BKPFILE $SOURCE --exclude=$EXCLUDES >> $CAMLOGS/$LOGFILE 2>&1 
if [[ $LOKBACKUP == 1 ]]; then
	echo -e "\033[1;32mLokale Archivierung abgeschlossen.\033[0m"
fi

#CleanUp-Lokal
if [[ $LOKBACKUP == 1 ]]; then
	if [[ $3 -ne 0 ]]; then
		find $BACKUPDIR/$BKPDELO -mtime +$3 -exec rm {} \;
		echo -e "\033[1;34mLokale Archive aelter als $3 Tage wurden geloescht.\033[0m"
	fi
fi

# LOG
echo ":Backup LOKAL : Ende   -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE
echo "" >> $CAMLOGS/$LOGFILE

# -------------------------------------------------------------------------------------------------------------------------
# Kennwortschutz?
# -------------------------------------------------------------------------------------------------------------------------
#GPG-Schutz
if [[ $GPGPASS == 1 ]]; then
	BKPFILE2=$BKPFILE
else
	BKPFILE2="$BKPFILE.gpg"
	echo ""
	echo -e "\033[1;31mArchivkennwort wird gesetzt..\033[0m"
	echo $4 | gpg -c --passphrase-fd 0 $BACKUPDIR/$BKPFILE
	echo ""
	#LOG
	echo ":GPG Schutz   : AKTIV   -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE
	echo "" >> $CAMLOGS/$LOGFILE
fi

# -------------------------------------------------------------------------------------------------------------------------
# Backup LAN
# -------------------------------------------------------------------------------------------------------------------------
if [[ $LANBACKUP == 1 ]]; then

	# LOG
	echo ":Backup LAN   : Start  -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE

	#mount -t smbfs -o username=$LANUSER,password=$LANPASS, $LANSHARE $MOUNTPATH
	cp $BACKUPDIR/$BKPFILE2 $MOUNTPATH/$BKPFILE2
	echo -e "\033[1;32mNetzwerk Archivierung abgeschlossen.\033[0m"
	#CleanUp-Lokal
	if [[ $3 -ne 0 ]]; then

		# LOG
		echo "" >> $CAMLOGS/$LOGFILE
		echo "Backup LAN   : Clean   -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE

		find $MOUNTPATH/$BKPDELO -mtime +$3 -exec rm {} \; >> $CAMLOGS/$LOGFILE 2>&1
		echo -e "\033[1;34mNetzwerk Archive aelter als $3 Tage wurden geloescht.\033[0m"
	fi
	# Umount
	# LOG
	echo ":Backup LAN   : umount -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE
	echo "" >> $CAMLOGS/$LOGFILE
	umount $MOUNTPATH >> $CAMLOGS/$LOGFILE
fi

# -------------------------------------------------------------------------------------------------------------------------
# Backup FTP
# -------------------------------------------------------------------------------------------------------------------------
if [[ $FTPBACKUP == 1 ]]; then
	# LOG
	echo ":Backup FTP   : Start  -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE
	ncftpput -m -u "$FTPUSER" -p "$FTPPASS" -P "$FTPPORT" "$FTPSERVER" "$FTPDIR" $BACKUPDIR/$BKPFILE2 2>> $CAMLOGS/$LOGFILE
	if [ ! "`grep 'B/s' $CAMLOGS/$LOGFILE`" = "" ] ; then
		echo -e "\033[1;32mFTP-Server Archivierung abgeschlossen.\033[0m"
		DISP="1"
	else
		echo -e "\033[1;31mFTP-Server Archivierung fehlgeschlagen, siehe LOG:\033[0m"
		echo -e "\033[1;36m`cat $CAMLOGS/$LOGFILE`\033[0m"			
	fi
	# LOG
	echo ":Backup FTP   : Ende   -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE
	echo "" >> $CAMLOGS/$LOGFILE
fi

# -------------------------------------------------------------------------------------------------------------------------
# Aufraeumen
# -------------------------------------------------------------------------------------------------------------------------
# Temporaeres Lokales Backup entfernen (falls nicht gewaehlt)
if [[ $LOKBACKUP == 0 ]]; then
	rm -f $BACKUPDIR/$BKPFILE
	#LOG
	echo ":Backup LOKAL : delete -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE
	echo "" >> $CAMLOGS/$LOGFILE
fi
# Loesche gpg.File
rm -f $BACKUPDIR/*.gpg

# -------------------------------------------------------------------------------------------------------------------------
# Ergebniss
# -------------------------------------------------------------------------------------------------------------------------
echo ""
echo "-------------------------------------------------------------------------------"
if [[ $LOKBACKUP == 1 ]]; then
echo "LOKAL : $BACKUPDIR/$BKPFILE"; fi
if [[ $LANBACKUP == 1 ]]; then
echo "LAN   : $LANSHARE/$BKPFILE2"; fi
if [[ $FTPBACKUP == 1 ]] && [[ $DISP == 1 ]]; then
echo "FTP   : $FTPSERVER:$FTPPORT$FTPDIR/$BKPFILE2"; fi
echo "-------------------------------------------------------------------------------"
echo ""

# -------------------------------------------------------------------------------------------------------------------------

# LOG
echo ":EXIT                  -> $(date "+%H:%M:%S")" >> $CAMLOGS/$LOGFILE

# -------------------------------------------------------------------------------------------------------------------------
# Mailnachricht
# -------------------------------------------------------------------------------------------------------------------------
if [ $BACKUPMAIL == 1 ]; then
	SUBJECT="IPC CS-Server $BACKUPART"
	echo "`hostname -f` hat ein $BACKUPART erstellt." > /tmp/mail
	echo "" >> /tmp/mail
	echo "Ziele:" >> /tmp/mail
	[[ $LOKBACKUP == 1 ]] && echo "LOKAL : $BACKUPDIR/$BKPFILE"  >> /tmp/mail
	[[ $LANBACKUP == 1 ]] && echo "LAN   : $LANSHARE/$BKPFILE2" >> /tmp/mail
	[[ $FTPBACKUP == 1 ]] && [[ $DISP == 1 ]] && echo "FTP   : $FTPSERVER:$FTPPORT$FTPDIR/$BKPFILE2" >> /tmp/mail
	echo "" >> /tmp/mail
	echo "" >> /tmp/mail
	echo "Logdatei im Anhang" >> /tmp/mail
	sed -e 's/$/\r/' /tmp/mail > /tmp/mail.txt
	sendEmail -f $SMTPFROM -t $SMTPTO -u "$SUBJECT" -o message-file=/tmp/mail.txt -a $CAMLOGS/$LOGFILE -s $SMTPSERVER -xu $SMTPUSER -xp $SMTPPASS > /dev/null 2>&1
	rm -f /tmp/mail && rm -f /tmp/mail.txt > /dev/null 2>&1
fi

# -------------------------------------------------------------------------------------------------------------------------

exit 0

